#ifndef CGP_H
#define CGP_H

#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <string>

using namespace std;

namespace CartesianGeneticProgramming {

#define WORST_FITNESS (1e+36f)

#define CGP_USES_PARAMETER_FLAG (0x1000)

//#define CGP_PROTOCOL_FUNCTIONS
#define CGP_PROTOCOL_FUNCTIONS_SMALL
//#define CGP_COMPACT_MATH_FUNCTIONS
//#define CGP_MATH_FUNCTIONS
//#define CGP_BIT_FUNCTIONS

//#define CGP_CLIP_TO_BYTE_OUTPUT
#define CGP_INTEGER_CONSTANTS

//#define USE_INTERNAL_SRAND
//#define NO_OPTIMIZATION

#define NO_CGP_SANITY_CHECKS

class CGP {
public:
	static int GetClipping() {
#ifdef CGP_CLIP_TO_BYTE_OUTPUT
		return 255;
#else
		return 0;
#endif
	}

	static int GetFunctionSet() {
#ifdef CGP_PROTOCOL_FUNCTIONS
		return 1;
#elif defined(CGP_COMPACT_MATH_FUNCTIONS)
		return 2;
#elif defined(CGP_MATH_FUNCTIONS)
		return 3;
#elif defined(CGP_BIT_FUNCTIONS)
		return 4;
#elif defined(CGP_PROTOCOL_FUNCTIONS_SMALL)
		return 5;
#else
#error
#endif
	}

	static int GetFunctionCount() {
		static int functionCount = -1;
		if(functionCount <= 0) {
			for(int i=255;i>=0;i--) {
				if(GetFunctionName(i)) {
					functionCount = i+1;
					break;
				}
			}
		}
		return functionCount;
	}

	static int GetFunctionInputCount(int index) {
#ifdef CGP_COMPACT_MATH_FUNCTIONS
			switch(((int)(*gp))) {
				case 0: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 1: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 2: // Multiply
					*vp = values[(int)gp[2]] * values[(int)gp[3]];
					break;
				case 3: // Divide (guarded)
					if(values[(int)gp[3]] <= 1e-1 && values[(int)gp[3]] >= -1e-1) {
						*vp = values[(int)gp[2]];
					} else {
						*vp = values[(int)gp[2]] / values[(int)gp[3]];
					}
					break;
				case 4: // Average
					*vp = (values[(int)gp[2]] + values[(int)gp[3]]) / 2;
					break;
			}
#elif defined(CGP_MATH_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Constant
					*vp = gp[1];
					break;
				case 1: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 2: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 3: // Add parameter
					*vp = gp[1] + values[(int)gp[2]];
					break;
				case 4: // Subtract parameter
					*vp = values[(int)gp[2]] - gp[1];
					break;
				case 5: // Negate
					*vp = -values[(int)gp[2]];
					break;
				case 6: // Multiply
					*vp = values[(int)gp[2]] * values[(int)gp[3]];
					break;
				case 7: // Divide (guarded)
					if(values[(int)gp[3]] <= 1e-1 && values[(int)gp[3]] >= -1e-1) {
						*vp = values[(int)gp[2]];
					} else {
						*vp = values[(int)gp[2]] / values[(int)gp[3]];
					}
					break;
				case 8: // Modulus (guarded)
					if(values[(int)gp[3]] < 0) {
						*vp = (float)(-(int)values[(int)gp[2]]);
					} else if(values[(int)gp[3]] < 1) {
						*vp = (float)((int)values[(int)gp[2]]);
					} else {
						*vp = (float)((int)values[(int)gp[2]] % (int)values[(int)gp[3]]);
					}
					break;
				case 9: // Average
					*vp = (values[(int)gp[2]] + values[(int)gp[3]]) / 2;
					break;
				case 10: // NOP
					*vp = values[(int)gp[2]];
					break;
			}
#elif defined(CGP_BIT_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Bitwise AND
					*vp = (int)values[(int)gp[2]] & (int)values[(int)gp[3]];
					break;
				case 1: // Bitwise OR
					*vp = (int)values[(int)gp[2]] | (int)values[(int)gp[3]];
					break;
				case 2: // Bitwise NOT
					*vp = ~(int)values[(int)gp[2]];
					break;
				case 3: // Constant
					*vp = gp[1] > 0? 1 : 0;
					break;
				case 4: // Add
					*vp = ((int)(values[(int)gp[2]] + values[(int)gp[3]])) & 1;
					break;
				case 5: // Subtract
					*vp = ((int)(values[(int)gp[2]] - values[(int)gp[3]])) & 1;
					break;
				case 6: // Add parameter
					*vp = ((int)(gp[1] + values[(int)gp[2]])) & 1;
					break;
				case 7: // Subtract parameter
					*vp = ((int)(values[(int)gp[2]] - gp[1])) & 1;
					break;
			}
#elif defined(CGP_PROTOCOL_FUNCTIONS)
			switch(index) {
				case 0: // Constant
					return CGP_USES_PARAMETER_FLAG;
				case 1: // Add
					return 2;
				case 2: // Subtract
					return 2;
				case 3: // Add parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 4: // Subtract parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 5: // Negate
					return 1;
				case 6: // NOP
					return 1;
				case 7: // Bitwise AND
					return 2;
				case 8: // Bitwise OR
					return 2;
				case 9: // Logical NOT
					return 1;
				case 10: // Logical AND
					return 2;
				case 11: // Logical OR
					return 2;
				case 12: // Greater than parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 13: // Greater than or equal to parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 14: // Greater than
					return 2;
				case 15: // Greater than or equal to
					return 2;
				case 16: // Equal to parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 17: // Equal to
					return 2;
				default:
					return 2;
			}
#elif defined(CGP_PROTOCOL_FUNCTIONS_SMALL)
			switch(index) {
				case 0: // Constant
					return CGP_USES_PARAMETER_FLAG;
				case 1: // Add
					return 2;
				case 2: // Subtract
					return 2;
				case 3: // Add parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 4: // Subtract parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 5: // Negate
					return 1;
				case 6: // NOP
					return 1;
				case 7: // Logical NOT
					return 1;
				case 8: // Logical AND
					return 2;
				case 9: // Logical OR
					return 2;
				case 10: // Greater than parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 11: // Greater than or equal to parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 12: // Greater than
					return 2;
				case 13: // Greater than or equal to
					return 2;
				case 14: // Equal to parameter
					return CGP_USES_PARAMETER_FLAG | 1;
				case 15: // Equal to
					return 2;
				default:
					return 2;
			}
#else
#error
#endif
	}

	static const char *GetFunctionName(int index) {
#ifdef CGP_COMPACT_MATH_FUNCTIONS
			switch(((int)(*gp))) {
				case 0: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 1: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 2: // Multiply
					*vp = values[(int)gp[2]] * values[(int)gp[3]];
					break;
				case 3: // Divide (guarded)
					if(values[(int)gp[3]] <= 1e-1 && values[(int)gp[3]] >= -1e-1) {
						*vp = values[(int)gp[2]];
					} else {
						*vp = values[(int)gp[2]] / values[(int)gp[3]];
					}
					break;
				case 4: // Average
					*vp = (values[(int)gp[2]] + values[(int)gp[3]]) / 2;
					break;
			}
#elif defined(CGP_MATH_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Constant
					*vp = gp[1];
					break;
				case 1: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 2: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 3: // Add parameter
					*vp = gp[1] + values[(int)gp[2]];
					break;
				case 4: // Subtract parameter
					*vp = values[(int)gp[2]] - gp[1];
					break;
				case 5: // Negate
					*vp = -values[(int)gp[2]];
					break;
				case 6: // Multiply
					*vp = values[(int)gp[2]] * values[(int)gp[3]];
					break;
				case 7: // Divide (guarded)
					if(values[(int)gp[3]] <= 1e-1 && values[(int)gp[3]] >= -1e-1) {
						*vp = values[(int)gp[2]];
					} else {
						*vp = values[(int)gp[2]] / values[(int)gp[3]];
					}
					break;
				case 8: // Modulus (guarded)
					if(values[(int)gp[3]] < 0) {
						*vp = (float)(-(int)values[(int)gp[2]]);
					} else if(values[(int)gp[3]] < 1) {
						*vp = (float)((int)values[(int)gp[2]]);
					} else {
						*vp = (float)((int)values[(int)gp[2]] % (int)values[(int)gp[3]]);
					}
					break;
				case 9: // Average
					*vp = (values[(int)gp[2]] + values[(int)gp[3]]) / 2;
					break;
				case 10: // NOP
					*vp = values[(int)gp[2]];
					break;
			}
#elif defined(CGP_BIT_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Bitwise AND
					*vp = (int)values[(int)gp[2]] & (int)values[(int)gp[3]];
					break;
				case 1: // Bitwise OR
					*vp = (int)values[(int)gp[2]] | (int)values[(int)gp[3]];
					break;
				case 2: // Bitwise NOT
					*vp = ~(int)values[(int)gp[2]];
					break;
				case 3: // Constant
					*vp = gp[1] > 0? 1 : 0;
					break;
				case 4: // Add
					*vp = ((int)(values[(int)gp[2]] + values[(int)gp[3]])) & 1;
					break;
				case 5: // Subtract
					*vp = ((int)(values[(int)gp[2]] - values[(int)gp[3]])) & 1;
					break;
				case 6: // Add parameter
					*vp = ((int)(gp[1] + values[(int)gp[2]])) & 1;
					break;
				case 7: // Subtract parameter
					*vp = ((int)(values[(int)gp[2]] - gp[1])) & 1;
					break;
			}
#elif defined(CGP_PROTOCOL_FUNCTIONS)
			switch(index) {
				case 0: // Constant
					return "C";
				case 1: // Add
					return "+";
				case 2: // Subtract
					return "-";
				case 3: // Add parameter
					return "+ C";
				case 4: // Subtract parameter
					return "- C";
				case 5: // Negate
					return "-";
				case 6: // NOP
					return "nop";
				case 7: // Bitwise AND
					return "&";
				case 8: // Bitwise OR
					return "|";
				case 9: // Logical NOT
					return "!";
				case 10: // Logical AND
					return "&&";
				case 11: // Logical OR
					return "||";
				case 12: // Greater than parameter
					return "> C";
				case 13: // Greater than or equal to parameter
					return ">= C";
				case 14: // Greater than
					return ">";
				case 15: // Greater than or equal to
					return ">=";
				case 16: // Equal to parameter
					return "== C";
				case 17: // Equal to
					return "==";
				default:
					return NULL;
			}
#elif defined(CGP_PROTOCOL_FUNCTIONS_SMALL)
			switch(index) {
				case 0: // Constant
					return "C";
				case 1: // Add
					return "+";
				case 2: // Subtract
					return "-";
				case 3: // Add parameter
					return "+ C";
				case 4: // Subtract parameter
					return "- C";
				case 5: // Negate
					return "-";
				case 6: // NOP
					return "nop";
				case 7: // Logical NOT
					return "!";
				case 8: // Logical AND
					return "&&";
				case 9: // Logical OR
					return "||";
				case 10: // Greater than parameter
					return "> C";
				case 11: // Greater than or equal to parameter
					return ">= C";
				case 12: // Greater than
					return ">";
				case 13: // Greater than or equal to
					return ">=";
				case 14: // Equal to parameter
					return "== C";
				case 15: // Equal to
					return "==";
				default:
					return NULL;
			}
#else
#error
#endif
	}
};

class RandomNumberSource {
public:

public:
	RandomNumberSource() {
	}

	float Next() {
		return (rand() & 8191) * (1.f/8192.f);
	}

	int NextInt(int range) {
		return rand() % range;
	}
};

class CGPIndividual {
public:
	// nr*nc*[function,parameter,input0,input1,...,inputNA-1], output0, output1,...,outputNO-1
	float *genome;
	int na, ni, no, nr, nc;
	float fitness;
	bool isTested;
	bool needsOptimization;
	bool *needed;
	int activeGeneCount;

	int randomSeed;
	int id;

public:
	CGPIndividual()
		: genome(NULL), na(0), ni(0), no(0), nr(0), nc(0), fitness(0), isTested(false), 
		randomSeed(1), id(0), needsOptimization(true), needed(NULL), activeGeneCount(0) {
	}

	CGPIndividual(int naToUse, int niToUse, int noToUse, int nrToUse, int ncToUse)
		: genome(NULL), na(naToUse), ni(niToUse), no(noToUse), nr(nrToUse), nc(ncToUse), fitness(0), isTested(false), 
		randomSeed(1), id(0), needsOptimization(true), needed(NULL), activeGeneCount(0)
	{
		genome = new float[no+(nr*nc)*(2+na)];
		randomSeed = (int)genome;
		needed = new bool[ni+nr*nc];
	}

	~CGPIndividual() {
		Cleanup();
	}

	void Cleanup() {
		if(genome) {
			delete [] genome;
			genome = NULL;
		}
		if(needed) {
			delete [] needed;
			needed = NULL;
		}
	}

	void Setup(int naToUse, int niToUse, int noToUse, int nrToUse, int ncToUse) {
		na = naToUse;
		ni = niToUse;
		no = noToUse;
		nr = nrToUse;
		nc = ncToUse;
		genome = new float[no+(nr*nc)*(2+na)];
		randomSeed = (int)genome;
		needed = new bool[ni+nr*nc];
	}

	void CopyFrom(const CGPIndividual &other) {
		memcpy(genome,other.genome,sizeof(float)*(no+(nr*nc)*(2+na)));
		needsOptimization = true;
		activeGeneCount = other.activeGeneCount;
		fitness = other.fitness;
	}

	void SetupFrom(const CGPIndividual &other) {
		na = other.na;
		ni = other.ni;
		no = other.no;
		nr = other.nr;
		nc = other.nc;
		genome = new float[no+(nr*nc)*(2+na)];
		CopyFrom(other);
		needed = new bool[ni+nr*nc];
	}

	void SetOutputSource(int outputIndex, int sourceIndex) {
		genome[(nr*nc)*(2+na)+outputIndex] = (float)sourceIndex;
	}

	void Randomize(RandomNumberSource &randomSource) {
#ifdef USE_INTERNAL_SRAND
		srand(randomSeed++);
#endif
		int r, c, a;
		float *gp = genome;
		for(c=0;c<nc;c++) {
			const int nci = c*nr + ni;
			for(r=0;r<nr;r++) {
				*gp++ = (float)randomSource.NextInt(CGP::GetFunctionCount());
#ifdef CGP_INTEGER_CONSTANTS
				*gp++ = (float)(randomSource.NextInt(21) - 10); // Set the parameter to be -10 to +10
#else
				*gp++ = (randomSource.Next() * 20) - 10; // Set the parameter to be -10 to +10
#endif
				for(a=0;a<na;a++) {
					*gp++ = (float)randomSource.NextInt(nci); // Point to an input or anything in any prior column
				}
			}
		}

		const int nrci = nr*nc + ni;
		for(int oi=0;oi<no;oi++) {
			*gp++ = (float)randomSource.NextInt(nrci);
		}
		needsOptimization = true;
	}

	void Save(const char *fileName) const {
		std::ofstream f;
		f.open(fileName,std::ios::out | std::ios::trunc);

		f << "functionSet " << CGP::GetFunctionSet() << std::endl;
		f << "fitness " << fitness << std::endl;
		f << "na " << na << std::endl;
		f << "ni " << ni << std::endl;
		f << "no " << no << std::endl;
		f << "nr " << nr << std::endl;
		f << "nc " << nc << std::endl;
		f << "genes" << std::endl;
		float *gp = genome;
		const int geneCount = nr * nc;
		int a, i;
		for(i=0;i<geneCount;i++) {
			// function,parameter,input0,input1,...,inputNA-1
			f << (int)gp[0] % CGP::GetFunctionCount() << " " << gp[1];
			gp += 2;
			for(a=0;a<na;a++,gp++) {
				f << " " << *gp;
			}
			f << std::endl;
		}
		for(i=0;i<no;i++,gp++) {
			f << *gp << std::endl;
		}
		f.close();
	}

	void Load(const char *fileName) {
		std::ifstream f;
		f.open(fileName,std::ios::in);

		int functionSet = 0;
		std::string prefix;
		while(!f.eof()) {
			f >> prefix;
			if(prefix == "functionSet") {
				f >> functionSet;
			} else if(prefix == "fitness") {
				f >> fitness;
			} else if(prefix == "na") {
				f >> na;
			} else if(prefix == "ni") {
				f >> na;
			} else if(prefix == "no") {
				f >> na;
			} else if(prefix == "nr") {
				f >> nr;
			} else if(prefix == "nc") {
				f >> nc;
			} else if(prefix == "genes") {
				Cleanup();
				Setup(na,ni,no,nr,nc);
				float *gp = genome;
				const int geneCount = nr * nc;
				int a, i;
				for(i=0;i<geneCount;i++) {
					// function,parameter,input0,input1,...,inputNA-1
					f >> gp[0] >> gp[1];
					gp[0] = (float)((int)gp[0] % CGP::GetFunctionCount());
					gp += 2;
					for(a=0;a<na;a++,gp++) {
						f >> *gp;
					}
				}
				for(i=0;i<no;i++,gp++) {
					f >> *gp;
				}
				// Done.
				break;
			}
		}
	}
};

class CGPMutator {
public:
	float rate;
	int limit;
	RandomNumberSource &randomSource;

public:
	CGPMutator(int limitToUse, RandomNumberSource &randomSourceToUse)
		: rate(0), limit(limitToUse), randomSource(randomSourceToUse) {
	}

	CGPMutator(float rateToUse, RandomNumberSource &randomSourceToUse)
		: rate(rateToUse), limit(-1), randomSource(randomSourceToUse) {
	}

	void ChangeRate(float rateToUse) {
		rate = rateToUse;
		limit = -1;
	}

	void Mutate(CGPIndividual *p, bool useHalfLimit=false) const {
		const int geneCount = p->nc*p->nr*(2+p->na) + p->no;
		int baseLimit = limit;
		// Calculate the limit if one was not provided
		if(rate > 0) {
			baseLimit = (int)(geneCount*rate);
			if(baseLimit < 1) {
				baseLimit = 1;
			}
		}

#ifdef USE_INTERNAL_SRAND
		srand(p->randomSeed++);
#endif
		p->isTested = false;
		p->needsOptimization = true;
		const int limitToUse = useHalfLimit? (baseLimit/2)+1 : baseLimit;
		for(int i=0;i<limitToUse;i++) {
			int index = randomSource.NextInt(geneCount);
			if(index < p->no) {
				// An output is to be mutated
				p->SetOutputSource(index,randomSource.NextInt(p->nc*p->nr + p->ni));
			} else {
				index -= p->no;
				int slot = index % (2+p->na);
				if(slot == 0) { // If changing the function
					// Mutate the function code
					p->genome[index] = (float)randomSource.NextInt(CGP::GetFunctionCount());
				} else if(slot == 1) { // If changing the parameter
					// Mutate the parameter to be -10 to +10
#ifdef CGP_INTEGER_CONSTANTS
					p->genome[index] = (float)(randomSource.NextInt(21) - 10); // Set the parameter to be -10 to +10
#else
					p->genome[index] = (randomSource.Next() * 20) - 10;
#endif
				} else { // If changing a wiring source
					// Mutate the wiring source. This must come from a previous column or an input.
					int c = (index/(2+p->na)) / p->nr;
					int newConnection = randomSource.NextInt(p->ni + (c*p->nr)); 
					//printf("Mutating index=%d, slot=%d, c=%d, old=%d, new=%d\n",index,slot,c,(int)p->genome[index],newConnection);
					p->genome[index] = (float)newConnection;
				}
			}
		}
	}
};

class CGPExecutor {
public:
	float *values;
	int startOfGenes;
	int startOfOutputs;
	int geneCount;
	int outputCount;

public:
	CGPExecutor()
		: values(NULL), startOfGenes(0), startOfOutputs(0), geneCount(0), outputCount(0) {
	}

	~CGPExecutor() {
		if(values != NULL) {
			delete [] values;
			values = NULL;
		}
	}

	void Setup(CGPIndividual *p) {
		geneCount = p->nr*p->nc;
		values = new float[p->ni + geneCount + p->no];
		startOfGenes = p->ni;
		startOfOutputs = startOfGenes + geneCount;
		outputCount = p->no;
	}

	void SetInput(int inputIndex, float value) const {
		values[inputIndex] = value;
	}

	void SetInputs(float *inputValues) const {
		float *vp = values;
		for(int i=0;i<startOfGenes;i++) {
			*vp++ = *inputValues++;
		}
	}

	float *GetOutputs() const {
		return values + startOfOutputs;
	}

	float GetOutput(int outputIndex) const {
		return values[startOfOutputs + outputIndex];
	}

	float *GetGenes() const {
		return values + startOfGenes;
	}

	void Optimize(CGPIndividual *p) const {
		//printf("Optimizing...\n");
		p->needsOptimization = false;
		const int geneSize = 2 + p->na;
		// Clear the flags
		for(int i=0;i<startOfOutputs;i++) {
			p->needed[i] = false;
		}
		const float *gp = p->genome + geneSize*geneCount; // start of the output genes
		// Mark everything needed by an output
		bool didPrint = false;
		for(int i=0;i<outputCount;i++,gp++) {
#ifndef NO_CGP_SANITY_CHECKS
			if((int)(*gp) >= startOfOutputs || (int)(*gp) < 0) {
				if(!didPrint) {
					didPrint = true;
					printf("Optimizing Output index is out of bounds: %d\n",(int)(*gp));
				}
				continue;
			}
#endif
			p->needed[(int)(*gp)] = true;
		}
		// Mark everything needed by each gene.  Work backwards from the outputs toward the inputs.
#ifndef NO_CGP_SANITY_CHECKS
		bool hasIssue = false;
#endif
		bool *np = p->needed + startOfOutputs - 1; // point to needed flag for last gene
		gp = p->genome + geneSize*(geneCount-1); // last non-output gene
		for(int i=0;i<geneCount;i++,gp-=geneSize,np--) {
#ifndef NO_CGP_SANITY_CHECKS
			hasIssue = false;
			if(gp[1] < -10 || gp[1] > +10) {
				if(!didPrint) {
					didPrint = true;
					printf("Optimizing Gene %d Parameter out of +/-10 bound: %f\n",i,gp[1]);
				}
				hasIssue = true;
			}
			if((int)gp[2] >= startOfOutputs || (int)gp[2] < 0) {
				if(!didPrint) {
					didPrint = true;
					printf("Optimizing Gene %d first wire out of bounds: %d\n",i,(int)gp[2]);
				}
				hasIssue = true;
			}
			if((int)gp[3] >= startOfOutputs || (int)gp[3] < 0) {
				if(!didPrint) {
					didPrint = true;
					printf("Optimizing Gene %d second wire out of bounds: %d\n",i,(int)gp[3]);
				}
				hasIssue = true;
			}
			if(hasIssue) {
				continue;
			}
#endif
			// Only consider genes that are needed by something previously considered
			if(!*np) {
				continue;
			}
#if defined(CGP_PROTOCOL_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Constant
					break;
				case 1: // Add
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 2: // Subtract
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 3: // Add parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 4: // Subtract parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 5: // Negate
					p->needed[(int)gp[2]] = true;
					break;
				case 6: // NOP
					p->needed[(int)gp[2]] = true;
					break;
				case 7: // Bitwise AND
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 8: // Bitwise OR
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 9: // Logical NOT
					p->needed[(int)gp[2]] = true;
					break;
				case 10: // Logical AND
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 11: // Logical OR
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 12: // Greater than parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 13: // Greater than or equal to parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 14: // Greater than
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 15: // Greater than or equal to
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 16: // Equal to parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 17: // Equal to
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				default: // NOP
					p->needed[(int)gp[2]] = true;
					break;
			}
#elif defined(CGP_PROTOCOL_FUNCTIONS_SMALL)
			switch(((int)(*gp))) {
				case 0: // Constant
					break;
				case 1: // Add
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 2: // Subtract
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 3: // Add parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 4: // Subtract parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 5: // Negate
					p->needed[(int)gp[2]] = true;
					break;
				case 6: // NOP
					p->needed[(int)gp[2]] = true;
					break;
				case 7: // Logical NOT
					p->needed[(int)gp[2]] = true;
					break;
				case 8: // Logical AND
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 9: // Logical OR
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 10: // Greater than parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 11: // Greater than or equal to parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 12: // Greater than
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 13: // Greater than or equal to
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				case 14: // Equal to parameter
					p->needed[(int)gp[2]] = true;
					break;
				case 15: // Equal to
					p->needed[(int)gp[2]] = true;
					p->needed[(int)gp[3]] = true;
					break;
				default: // NOP
					p->needed[(int)gp[2]] = true;
					break;
			}
#else
#error
#endif
		}
		// Count the number of genes needed.
		p->activeGeneCount = 0;
		for(int i=0;i<startOfOutputs;i++) {
			if(p->needed[i]) {
				p->activeGeneCount++;
			}
		}
	}

	void Execute(CGPIndividual *p) const {
		if(p->needsOptimization) {
			Optimize(p);
		}
		const float *gp = p->genome;
		const int geneSize = 2 + p->na;
		// Execute the genes
		float *vp = values + startOfGenes;
		bool *np = p->needed + startOfGenes;
#ifndef NO_CGP_SANITY_CHECKS
		bool hasIssue = false;
#endif
		for(int i=0;i<geneCount;i++,gp+=geneSize,vp++,np++) {
#ifndef NO_CGP_SANITY_CHECKS
			hasIssue = false;
			if(gp[1] < -10 || gp[1] > +10) {
				printf("Gene Parameter out of +/-10 bound: %f\n",gp[1]);
				hasIssue = true;
			}
			if((int)gp[2] >= startOfOutputs || (int)gp[2] < 0) {
				printf("Gene first wire out of bounds: %d\n",(int)gp[2]);
				hasIssue = true;
			}
			if((int)gp[3] >= startOfOutputs || (int)gp[3] < 0) {
				printf("Gene second wire out of bounds: %d\n",(int)gp[3]);
				hasIssue = true;
			}
			if(hasIssue) {
				*vp = 0;
				continue;
			}
#endif
			if(!*np) {
#ifndef NO_OPTIMIZATION
				*vp = 0;
				continue;
#endif
			}
#ifdef CGP_COMPACT_MATH_FUNCTIONS
			switch(((int)(*gp))) {
				case 0: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 1: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 2: // Multiply
					*vp = values[(int)gp[2]] * values[(int)gp[3]];
					break;
				case 3: // Divide (guarded)
					if(values[(int)gp[3]] <= 1e-1 && values[(int)gp[3]] >= -1e-1) {
						*vp = values[(int)gp[2]];
					} else {
						*vp = values[(int)gp[2]] / values[(int)gp[3]];
					}
					break;
				case 4: // Average
					*vp = (values[(int)gp[2]] + values[(int)gp[3]]) / 2;
					break;
			}
#elif defined(CGP_MATH_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Constant
					*vp = gp[1];
					break;
				case 1: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 2: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 3: // Add parameter
					*vp = gp[1] + values[(int)gp[2]];
					break;
				case 4: // Subtract parameter
					*vp = values[(int)gp[2]] - gp[1];
					break;
				case 5: // Negate
					*vp = -values[(int)gp[2]];
					break;
				case 6: // Multiply
					*vp = values[(int)gp[2]] * values[(int)gp[3]];
					break;
				case 7: // Divide (guarded)
					if(values[(int)gp[3]] <= 1e-1 && values[(int)gp[3]] >= -1e-1) {
						*vp = values[(int)gp[2]];
					} else {
						*vp = values[(int)gp[2]] / values[(int)gp[3]];
					}
					break;
				case 8: // Modulus (guarded)
					if(values[(int)gp[3]] < 0) {
						*vp = (float)(-(int)values[(int)gp[2]]);
					} else if(values[(int)gp[3]] < 1) {
						*vp = (float)((int)values[(int)gp[2]]);
					} else {
						*vp = (float)((int)values[(int)gp[2]] % (int)values[(int)gp[3]]);
					}
					break;
				case 9: // Average
					*vp = (values[(int)gp[2]] + values[(int)gp[3]]) / 2;
					break;
				case 10: // NOP
					*vp = values[(int)gp[2]];
					break;
			}
#elif defined(CGP_BIT_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Bitwise AND
					*vp = (int)values[(int)gp[2]] & (int)values[(int)gp[3]];
					break;
				case 1: // Bitwise OR
					*vp = (int)values[(int)gp[2]] | (int)values[(int)gp[3]];
					break;
				case 2: // Bitwise NOT
					*vp = ~(int)values[(int)gp[2]];
					break;
				case 3: // Constant
					*vp = gp[1] > 0? 1 : 0;
					break;
				case 4: // Add
					*vp = ((int)(values[(int)gp[2]] + values[(int)gp[3]])) & 1;
					break;
				case 5: // Subtract
					*vp = ((int)(values[(int)gp[2]] - values[(int)gp[3]])) & 1;
					break;
				case 6: // Add parameter
					*vp = ((int)(gp[1] + values[(int)gp[2]])) & 1;
					break;
				case 7: // Subtract parameter
					*vp = ((int)(values[(int)gp[2]] - gp[1])) & 1;
					break;
			}
#elif defined(CGP_PROTOCOL_FUNCTIONS)
			switch(((int)(*gp))) {
				case 0: // Constant
					*vp = gp[1];
					break;
				case 1: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 2: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 3: // Add parameter
					*vp = gp[1] + values[(int)gp[2]];
					break;
				case 4: // Subtract parameter
					*vp = values[(int)gp[2]] - gp[1];
					break;
				case 5: // Negate
					*vp = -values[(int)gp[2]];
					break;
				case 6: // NOP
					*vp = values[(int)gp[2]];
					break;
				case 7: // Bitwise AND
					*vp = (float)((int)values[(int)gp[2]] & (int)values[(int)gp[3]]);
					break;
				case 8: // Bitwise OR
					*vp = (float)((int)values[(int)gp[2]] | (int)values[(int)gp[3]]);
					break;
				case 9: // Logical NOT
					*vp = !(int)values[(int)gp[2]];
					break;
				case 10: // Logical AND
					*vp = (int)values[(int)gp[2]] && (int)values[(int)gp[3]];
					break;
				case 11: // Logical OR
					*vp = (int)values[(int)gp[2]] || (int)values[(int)gp[3]];
					break;
				case 12: // Greater than parameter
					*vp = values[(int)gp[2]] > gp[1];
					break;
				case 13: // Greater than or equal to parameter
					*vp = values[(int)gp[2]] >= gp[1];
					break;
				case 14: // Greater than
					*vp = values[(int)gp[2]] > values[(int)gp[3]];
					break;
				case 15: // Greater than or equal to
					*vp = values[(int)gp[2]] >= values[(int)gp[3]];
					break;
				case 16: // Equal to parameter
					*vp = values[(int)gp[2]] == gp[1];
					break;
				case 17: // Equal to
					*vp = values[(int)gp[2]] == values[(int)gp[3]];
					break;
				default: // NOP
					*vp = values[(int)gp[2]];
					break;
			}
#elif defined(CGP_PROTOCOL_FUNCTIONS_SMALL)
			switch(((int)(*gp))) {
				case 0: // Constant
					*vp = gp[1];
					break;
				case 1: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 2: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 3: // Add parameter
					*vp = gp[1] + values[(int)gp[2]];
					break;
				case 4: // Subtract parameter
					*vp = values[(int)gp[2]] - gp[1];
					break;
				case 5: // Negate
					*vp = -values[(int)gp[2]];
					break;
				case 6: // NOP
					*vp = values[(int)gp[2]];
					break;
				case 7: // Logical NOT
					*vp = !(int)values[(int)gp[2]];
					break;
				case 8: // Logical AND
					*vp = (int)values[(int)gp[2]] && (int)values[(int)gp[3]];
					break;
				case 9: // Logical OR
					*vp = (int)values[(int)gp[2]] || (int)values[(int)gp[3]];
					break;
				case 10: // Greater than parameter
					*vp = values[(int)gp[2]] > gp[1];
					break;
				case 11: // Greater than or equal to parameter
					*vp = values[(int)gp[2]] >= gp[1];
					break;
				case 12: // Greater than
					*vp = values[(int)gp[2]] > values[(int)gp[3]];
					break;
				case 13: // Greater than or equal to
					*vp = values[(int)gp[2]] >= values[(int)gp[3]];
					break;
				case 14: // Equal to parameter
					*vp = values[(int)gp[2]] == gp[1];
					break;
				case 15: // Equal to
					*vp = values[(int)gp[2]] == values[(int)gp[3]];
					break;
				default: // NOP
					*vp = values[(int)gp[2]];
					break;
			}
#else
#error
			switch(((int)(*gp))) {
				case 0: // Bitwise AND
					*vp = (int)values[(int)gp[2]] & (int)values[(int)gp[3]];
					break;
				case 1: // Bitwise OR
					*vp = (int)values[(int)gp[2]] | (int)values[(int)gp[3]];
					break;
				case 2: // Bitwise NOT
					*vp = ~(int)values[(int)gp[2]];
					break;
				case 3: // Logical AND
					*vp = (int)values[(int)gp[2]] && (int)values[(int)gp[3]];
					break;
				case 4: // Logical OR
					*vp = (int)values[(int)gp[2]] || (int)values[(int)gp[3]];
					break;
				case 5: // Constant
					*vp = gp[1];
					break;
				case 6: // Add
					*vp = values[(int)gp[2]] + values[(int)gp[3]];
					break;
				case 7: // Subtract
					*vp = values[(int)gp[2]] - values[(int)gp[3]];
					break;
				case 8: // Add parameter
					*vp = gp[1] + values[(int)gp[2]];
					break;
				case 9: // Subtract parameter
					*vp = values[(int)gp[2]] - gp[1];
					break;
				case 10: // Negate
					*vp = -values[(int)gp[2]];
					break;
				case 11: // Multiply
					*vp = values[(int)gp[2]] * values[(int)gp[3]];
					break;
				case 12: // Divide (guarded)
					if(values[(int)gp[3]] <= 1e-1 && values[(int)gp[3]] >= -1e-1) {
						*vp = values[(int)gp[2]];
					} else {
						*vp = values[(int)gp[2]] / values[(int)gp[3]];
					}
					break;
				case 13: // Modulus (guarded)
					if(values[(int)gp[3]] < 0) {
						*vp = -(int)values[(int)gp[2]];
					} else if(values[(int)gp[3]] < 1) {
						*vp = (int)values[(int)gp[2]];
					} else {
						*vp = (int)values[(int)gp[2]] % (int)values[(int)gp[3]];
					}
					break;
				case 14: // Average
					*vp = (values[(int)gp[2]] + values[(int)gp[3]]) / 2;
					break;
			}
#endif
		}
		// Populate the outputs
		for(int i=0;i<outputCount;i++,vp++,gp++) {
#ifndef NO_CGP_SANITY_CHECKS
			if((int)(*gp) >= startOfOutputs || (int)(*gp) < 0) {
				printf("Output index is out of bounds: %d\n",(int)(*gp));
				*vp = 0;
				continue;
			} else {
				//printf("output[%d] from (%d) = %f & 255 = %f\n",i,(int)(*gp),values[(int)(*gp)],(float)((int)values[(int)(*gp)] & 255));
			}
#endif
#ifdef CGP_CLIP_TO_BYTE_OUTPUT
			*vp = (float)((int)values[(int)(*gp)] & 255);
#else
			*vp = values[(int)(*gp)];
#endif
		}
	}
};

class TestCases {
public:
	float *testData;
	int inputCount;
	int outputCount;
	int testSize;
	int howManyTests;
	int populationSize;

public:
	TestCases(int inputCountToUse, int outputCountToUse, int howManyTestsToUse, int populationSizeToUse)
			: testData(NULL), inputCount(inputCountToUse), outputCount(outputCountToUse), howManyTests(howManyTestsToUse), populationSize(populationSizeToUse) {
		testSize = inputCount + outputCount + populationSize;
		testData = new float[howManyTests * testSize];
	}

	~TestCases() {
		if(testData != NULL) {
			delete [] testData;
			testData = NULL;
		}
	}

	float *GetInputs(int testIndex) const {
		return testData + (testIndex * testSize);
	}

	float *GetOutputs(int testIndex) const {
		return testData + (testIndex * testSize) + inputCount;
	}

	void SetFitness(int testIndex, int individualIndex, float fitness) const {
		testData[testIndex * testSize + inputCount + outputCount + individualIndex] = fitness;
	}

	float GetFitness(int testIndex, int individualIndex) const {
		return testData[testIndex * testSize + inputCount + outputCount + individualIndex];
	}

	void CopyIndividualResults(int sourceIndex, int destIndex) const {
		for(int ti=0;ti<howManyTests;ti++) {
			SetFitness(ti,destIndex,GetFitness(ti,sourceIndex));
		}
	}

	float GetMaxFitness() const {
		float *fp = testData + inputCount + outputCount;
		float maxFitness = *fp;
		int pi;
		for(int ti=0;ti<howManyTests;ti++,fp+=inputCount+outputCount) {
			for(pi=0;pi<populationSize;pi++,fp++) {
				if(maxFitness < *fp) {
					maxFitness = *fp;
				}
			}
		}
		return maxFitness;
	}

	float GetMaxFitness(int individualIndex) const {
		float *fp = testData + inputCount + outputCount + individualIndex;
		float maxFitness = *fp;
		for(int ti=0;ti<howManyTests;ti++,fp+=testSize) {
			if(maxFitness < *fp) {
				maxFitness = *fp;
			}
		}
		return maxFitness;
	}
};

class CGPPopulation {
public:
	CGPIndividual *population;
	int populationSize;
	int na, ni, no, nr, nc;
	int nextID;

public:
	CGPPopulation(int populationSizeToUse, int naToUse, int niToUse, int noToUse, int nrToUse, int ncToUse)
		: population(NULL), populationSize(populationSizeToUse),
		na(naToUse), ni(niToUse), no(noToUse), nr(nrToUse), nc(ncToUse), nextID(0) {
		population = new CGPIndividual[populationSize];
		for(int i=0;i<populationSize;i++) {
			population[i].id = nextID+i;
			population[i].Setup(na,ni,no,nr,nc);
		}
		nextID = populationSize+1;
	}

	~CGPPopulation() {
		if(population) {
			delete [] population;
			population = NULL;
		}
	}

	void Randomize(RandomNumberSource &randomSource) {
		#pragma omp parallel for
		for(int pi=0;pi<populationSize;pi++) {
			population[pi].Randomize(randomSource);
		}
	}

	void Execute(TestCases &tests, bool testAllCases=true) const {
		#pragma omp parallel for
		for(int pi=0;pi<populationSize;pi++) {
			CGPIndividual *p = &population[pi];
			if(p->isTested) {
				continue;
			}
			CGPExecutor ex;
			ex.Setup(p);
			float fitness = 0, diff, t;
			for(int ti=0;ti<tests.howManyTests;ti++) {
				ex.SetInputs(tests.GetInputs(ti));
				ex.Execute(p);
				diff = 0;
				float *desiredOutput = tests.GetOutputs(ti);
				float *actualOutput = ex.GetOutputs();
				for(int oi=0;oi<p->no;oi++,desiredOutput++,actualOutput++) {
					t = (*desiredOutput - *actualOutput) * (*desiredOutput - *actualOutput);
					//printf("t[%d][%d][%d] = %f, desired=%f, actual=%f\n",pi,ti,oi,t,*desiredOutput,*actualOutput);
					diff += t;
				}
				//printf("diff[%d][%d]=%f\n",pi,ti,diff);
				tests.SetFitness(ti,pi,diff);
				fitness += diff;
				if(!testAllCases && fitness > population[0].fitness) {
					break;
				}
			}
			p->fitness = fitness;
			p->isTested = true;
			//printf("fitness[%d]=%f\n",pi,p->fitness);
		}
	}

	void PromoteBest(const CGPMutator &mutator, const TestCases *tests) {
		float bestFitness = population[0].fitness;
		int bestIndex = 0;
		for(int i=1;i<populationSize;i++) {
			if(population[i].fitness <= bestFitness) {
				bestFitness = population[i].fitness;
				bestIndex = i;
			}
		}
		if(bestIndex != 0) {
			population[0].CopyFrom(population[bestIndex]);
			population[0].fitness = population[bestIndex].fitness;
			population[0].isTested = true;
			population[0].id = population[bestIndex].id;
			if(tests) {
				tests->CopyIndividualResults(bestIndex,0);
			}
		}
		#pragma omp parallel for
		for(int i=1;i<populationSize;i++) {
			population[i].CopyFrom(population[0]);
			population[i].id = nextID + i - 1;
			mutator.Mutate(&population[i]);
			/*
			if(i==1) {
				// Mutate the first child at half the normal rate.  The hope is that if no other child at least
				// matches the parent, this one will.  Since the selection for fitest individual gives preference
				// to higher indexed children, children mutating at a normal rate but with the same score as the
				// parent will still drive the evolution as normal.
				mutator.Mutate(&population[i],true);
			} else {
				mutator.Mutate(&population[i]);
			}
			*/
		}
		nextID += populationSize-1;
	}

	void PromoteBest(const CGPMutator &mutator, const TestCases &tests) {
		PromoteBest(mutator,&tests);
	}

	void PromoteBest(const CGPMutator &mutator) {
		PromoteBest(mutator,NULL);
	}
};

};

#endif
