#include <string.h>
#include <string>
#include "CGPGraphViz.h"

CGPGraphViz::CGPGraphViz() {
	// Nothing to do here
}

CGPGraphViz::~CGPGraphViz() {
	// Nothing to do here
}

void CGPGraphViz::Convert(const char *inputCGPFileName, const char *outputDottyFileName, const vector<string> &inputNames, const vector<string> &outputNames) {
	CGPIndividual program;
	program.Load(inputCGPFileName);
	Create(program,outputDottyFileName,inputNames,outputNames);
}

void CGPGraphViz::Create(CGPIndividual &program, const char *outputDottyFileName, const vector<string> &inputNames, const vector<string> &outputNames) {
	if(program.needsOptimization) {
		CGPExecutor ex;
		ex.Setup(&program);
		ex.Optimize(&program);
	}
	std::ofstream f;
	f.open(outputDottyFileName,std::ios::out | std::ios::trunc);
	f << "digraph g {" << std::endl;
	f << "graph [rankdir = \"LR\" ];" << std::endl;

	int nodeCounter = 0;
	const bool *np = program.needed;

	// Write inputs
	/*
	f << "subgraph inputs {" << std::endl;
	f << "label=\"Inputs\"" << std::endl;
	for(int i=0;i<inputNames.size() && i<program.ni;i++,nodeCounter++,np++) {
		char buffer[50];
		sprintf(buffer,"n%d",nodeCounter);
		f << buffer << " [label=\"" << inputNames[i] << "\"];" << std::endl;
	}
	f << "}" << std::endl;
	*/
	f << "subgraph clusterInputs {" << std::endl << "label=\"Inputs\"" << std::endl;
	f << "nInputs [ shape=\"record\"" << std::endl << "label=\"";
	for(int i=0;i<inputNames.size() && i<program.ni;i++,nodeCounter++,np++) {
		if(i > 0) {
			f << "|";
		}
		char buffer[50];
		sprintf(buffer,"n%d",nodeCounter);
		f << "<" << buffer << "> " << inputNames[i];
	}
	f << "\"];" << std::endl << "}" << std::endl;

	// Write genes, but only the genes that are needed by an output
	f << "subgraph clusterGenes {" << std::endl;
	f << "label=\"Genes\"" << std::endl;
	// nr*nc*[function,parameter,input0,input1,...,inputNA-1], output0, output1,...,outputNO-1
	const float *gp = program.genome;
	const int geneSize = 2 + program.na;
	const int geneCount = program.nr * program.nc;
	const int functionCount = CGP::GetFunctionCount();
	for(int i=0;i<geneCount;i++,gp+=geneSize,nodeCounter++,np++) {
		if(!*np) {
			continue;
		}
		char buffer[50];
		sprintf(buffer,"n%d",nodeCounter);
		int functionIndex = ((int)gp[0]) % functionCount;
		int inputCount = CGP::GetFunctionInputCount(functionIndex);
		bool needsParameter = (inputCount & CGP_USES_PARAMETER_FLAG) != 0;
		inputCount = inputCount & ~CGP_USES_PARAMETER_FLAG;
		if(needsParameter) {
			double roundedParameter = (double)(((int)(gp[1] * 100)) / 100.0);
			f << buffer << " [label=\"" << CGP::GetFunctionName(functionIndex) << " " << roundedParameter << "\"];" << std::endl;
		} else {
			f << buffer << " [label=\"" << CGP::GetFunctionName(functionIndex) << "\"];" << std::endl;
		}
		for(int j=0;j<inputCount;j++) {
			int sourceIndex = (int)gp[2+j];
			if(sourceIndex < program.ni) {
				f << "nInputs:n" << sourceIndex << " -> " << buffer << ";" << std::endl;
			} else {
				f << "n" << sourceIndex << " -> " << buffer << ";" << std::endl;
			}
		}
	}
	f << "}" << std::endl;

	// Write outputs
	/*
	f << "subgraph outputs {" << std::endl;
	f << "label=\"Outputs\"" << std::endl;
	for(int i=0;i<outputNames.size() && i<program.no;i++,nodeCounter++,gp++) {
		char buffer[50];
		sprintf(buffer,"n%d",nodeCounter);
		f << buffer << " [label=\"" << outputNames[i] << "\"];" << std::endl;
		f << "n" << (int)(*gp) << " -> " << buffer << ";" << std::endl;
	}
	f << "}" << std::endl;
	*/
	f << "subgraph clusterOutputs {" << std::endl << "label=\"Outputs\"" << std::endl;
	f << "nOutputs [ shape=\"record\"" << std::endl << "label=\"";
	int nodeCounterBeforeOutputs = nodeCounter;
	for(int i=0;i<outputNames.size() && i<program.no;i++,nodeCounter++) {
		if(i > 0) {
			f << "|";
		}
		char buffer[50];
		sprintf(buffer,"n%d",nodeCounter);
		f << "<" << buffer << "> " << outputNames[i];
	}
	f << "\"];" << std::endl << "}" << std::endl;
	nodeCounter = nodeCounterBeforeOutputs;
	for(int i=0;i<outputNames.size() && i<program.no;i++,nodeCounter++,gp++) {
		char buffer[50];
		sprintf(buffer,"n%d",nodeCounter);
		int sourceIndex = (int)(*gp);
		if(sourceIndex < program.ni) {
			f << "nInputs:n" << sourceIndex << " -> nOutputs:" << buffer << ";" << std::endl;
		} else {
			f << "n" << sourceIndex << " -> nOutputs:" << buffer << ";" << std::endl;
		}
	}

	// Done
	f << "}" << std::endl;
	f.close();
}
