#ifndef CircularQueue_h
#define CircularQueue_h

template <class T, int maxSize>
class CircularQueue {
public:
	int head, tail, count, minCount, maxCount, underFlowCount, overFlowCount;
	T data[maxSize];

	int HasErrors() const { return underFlowCount || overFlowCount; }

	void Print(const char *prefix) {
		printf("%s.count=%d\n",prefix,count);
		//printf("%s.minCount=%d\n",prefix,minCount);
		//printf("%s.maxCount=%d\n",prefix,maxCount);
		//printf("%s.underFlowCount=%d\n",prefix,underFlowCount);
		//printf("%s.overFlowCount=%d\n",prefix,overFlowCount);
	}

	void PrintCSVHeader(FILE *fp, const char *prefix) {
		fprintf(fp,"%s_count,",prefix);

		fprintf(fp,"%s_minCount,",prefix);
		fprintf(fp,"%s_maxCount,",prefix);
		//fprintf(fp,"%s_underFlowCount,",prefix);
		//fprintf(fp,"%s_overFlowCount,",prefix);
	}

	void PrintCSVData(FILE *fp) {
		fprintf(fp,"%d,",count);

		fprintf(fp,"%d,",minCount);
		fprintf(fp,"%d,",maxCount);
		//fprintf(fp,"%d,",underFlowCount);
		//fprintf(fp,"%d,",overFlowCount);
	}

	void Setup() {
		head = tail = minCount = maxCount = count = 0;
		underFlowCount = overFlowCount = 0;
	}

	void Setup(T lowValue, T highValue) {
		head = 0;
		for(;lowValue<=highValue;lowValue++) {
			data[tail] = lowValue;
			tail++;
			count++;
		}
		minCount = maxCount = count;
		underFlowCount = overFlowCount = 0;
	}

	void Push(T value) {
		int next = (tail + 1) % maxSize;
		if(next != head) {
			data[tail] = value;
			tail = next;
			count++;
			if(count > maxCount) {
				maxCount = count;
			}
		} else {
			overFlowCount++;
		}
	}

	T Pop() {
		if(head != tail) {
			int resultIndex = head;
			count--;
			if(count < minCount) {
				minCount = count;
			}
			head = (head + 1) % maxSize;
			return data[resultIndex];
		} else {
			underFlowCount++;
		}
		return 0;
	}

	T Peek(int index) {
		return data[(head + index) % maxSize];
	}

	void Remove(int index) {
		if(index == 0) {
			pop();
		} else if(index == (count-1)) {
			// Remove from the end
			tail = (tail - 1 + maxSize) % maxSize;
			count--;
			if(count < minCount) {
				minCount = count;
			}
		} else {
			// Remove from the middle
			count--;
			if(count < minCount) {
				minCount = count;
			}
			for(;index<count;index++) {
				data[(head+index)%maxSize] = data[(head+index+1)%maxSize];
			}
			tail = (tail - 1 + maxSize) % maxSize;
		}
	}

	bool HasData() const { return head != tail; }
	bool IsEmpty() const { return head == tail; }
	bool HasRoom() const { return ((tail + 1) % maxSize) != head; }
};

template <class T, int maxSize>
class CircularQueueWithoutStats {
public:
	int head, tail, count;
	T data[maxSize];

	void Setup() {
		head = tail = count = 0;
	}

	void Setup(T lowValue, T highValue) {
		head = 0;
		for(;lowValue<=highValue;lowValue++) {
			data[tail] = lowValue;
			tail++;
			count++;
		}
	}

	void Push(T value) {
		int next = (tail + 1) % maxSize;
		if(next != head) {
			data[tail] = value;
			tail = next;
			count++;
		}
	}

	T Pop() {
		if(head != tail) {
			int resultIndex = head;
			count--;
			head = (head + 1) % maxSize;
			return data[resultIndex];
		}
		return 0;
	}

	T Peek(int index) {
		return data[(head + index) % maxSize];
	}

	void Remove(int index) {
		if(index == 0) {
			pop();
		} else if(index == (count-1)) {
			// Remove from the end
			tail = (tail - 1 + maxSize) % maxSize;
			count--;
		} else {
			// Remove from the middle
			count--;
			for(;index<count;index++) {
				data[(head+index)%maxSize] = data[(head+index+1)%maxSize];
			}
			tail = (tail - 1 + maxSize) % maxSize;
		}
	}

	bool HasData() const { return head != tail; }
	bool IsEmpty() const { return head == tail; }
	bool HasRoom() const { return ((tail + 1) % maxSize) != head; }
};

#endif
